#!/usr/bin/python3

# Command line focus control by image defocus in pixels or arcseconds

# Moore Observatory focuser control
# Copyright (c) 2012-2018 John Kielkopf
# kielkopf@louisville.edu

# Date: May 6, 2018
# Version: 2.0

# This file is part of PyFocus


#Basic imports

import sys
from time import sleep       # for delays 

#Phidget imports

from Phidget22.Devices.Encoder import *
from Phidget22.Devices.DCMotor import *
from Phidget22.PhidgetException import *
from Phidget22.Phidget import *

# Useful global variables

global motor_count
global motor_velocity
global focus_position
global focus_pix
global focus_arcsec

motor_count = 0
motor_velocity = 0
focus_position = 0
focus_pix = 0
focus_arcsec = 0

# Set to true for diagnostic messages

diagnostics = True

#
# Change these for specific telescope/focuser/motor combination
#

# Location of status file

#focus_status_file = "/usr/local/observatory/status/telfocus"

focus_status_file = './telfocus'


# Begin user installation  parameters
#
#
# Phidgets motor on the CDK20 telescopes
#
# Motor RPM at 100% duty cycle: 4000.
# Motor RPM speed factor: 0.5 estimated
# Motor shaft encoder counts/turn: 360.
# Motor gear reduction: 139. + 184./1221. = 139.150696151
# Focuser screw pitch: 32 tpi
# Focuser translation scale: 793.75 microns / shaft revolution
# Motor counts per micron focuser motion: 63.111
# Focal ratio: f/6.8
# Pixel size: 9 microns
# Secondary motion magnification: 1.0 
#
# Set CDK20 motor and focuser parameters
#
# counts_per_revolution = 360.
# revolutions_per_second = 0.5*4000./60.
# counts_per_second = counts_per_revolution * revolutions_per_second
# counts_per_micron = 63.11
# delta_count_max = 1000.*counts_per_micron
#
# Set CDK20 telescope parameters
#
# f_ratio = 6.8
# f_mag = 1.0
# pixel_size = 9.0
# arcsecond_per_pixel = 0.53
#
#
# Pittman 9237S011-R1 motor on the RC telescopes
#
# Set RC motor and focuser parameters
#
# Screw pitch: 20 tpi
# Motor encoder: 500 counts per turn
# Motor gearing: 1:1
# No-load speed: 4370 rpm 
#
# counts_per_revolution = 500.
# revolutions_per_second = 20.
# counts_per_second = counts_per_revolution * revolutions_per_second
# counts_per_micron = 0.3937
# delta_count_max = 1000.*counts_per_micron
#
# Set RC telescope parameters
#
# f_ratio = 8.0
# f_mag = 4.0
# pixel_size = 9.0
# arcsecond_per_pixel = 0.39


# Set motor and focuser parameters

counts_per_revolution = 360.
revolutions_per_second = 0.5*4000./60.
counts_per_second = counts_per_revolution * revolutions_per_second
counts_per_micron = 63.11
delta_count_max = 1000.*counts_per_micron

# Set telescope parameters

f_ratio = 6.8
f_mag = 1.0
pixel_size = 9.0
arcsecond_per_pixel = 0.53

# Set image scale for pixel and arcseond defocus units

counts_per_pixel =  pixel_size*f_ratio*counts_per_micron/f_mag
counts_per_arcsec = counts_per_pixel/arcsecond_per_pixel


#
# End of user defined installation parameters
#


# Check that the user supplied a change in pixels


if len(sys.argv) == 1:
  print (" ")
  sys.exit("Usage: focus_change_pixels.py pixels (Signed change as an argument)")
elif len(sys.argv) == 2:
  input_change_pix = float(sys.argv[1])
else:
  print (" ")
  sys.exit("Usage: focus_change_pixels.py pixels (Provide signed change as an argument)")



  
# Look up the saved last position of the focuser in microns if any
# Assign current encoder count to match or else set to zero

try: 
  focus_status_fp = open(focus_status_file,'r')
  focus_str = focus_status_fp.read(40)
  focus_status_fp.close()
  motor_count = float(focus_str)
except:
  print (focus_status_file+" is missing or empty")
  print ("Setting focus position to zero")
  print (" ")
  motor_count = 0.

# Set the initial focus position from the motor count

focus_position = motor_count/counts_per_micron


# Set the initial image  focus_pix in pixels and  focus_arcsec in arcseconds  

focus_pix = abs(motor_count)/counts_per_pixel
focus_arcsec = abs(motor_count)/counts_per_arcsec

# Subsequently these values will be updated by the encoder interrupt

    

# Setup motor

try:
  ch_motor = DCMotor()
except RuntimeError as e:
  print("Runtime Exception %s" % e.details)
  print("Press Enter to Exit...\n")
  readin = sys.stdin.read(1)
  exit(1)

def DCMotorAttached(e):
  try:
    attached = e
    #print("\nAttach Event Detected (Information Below)")
    #print("===========================================")
    #print("Library Version: %s" % attached.getLibraryVersion())
    #print("Serial Number: %d" % attached.getDeviceSerialNumber())
    #print("Channel: %d" % attached.getChannel())
    #print("Channel Class: %s" % attached.getChannelClass())
    #print("Channel Name: %s" % attached.getChannelName())
    #print("Device ID: %d" % attached.getDeviceID())
    #print("Device Version: %d" % attached.getDeviceVersion())
    #print("Device Name: %s" % attached.getDeviceName())
    #print("Device Class: %d" % attached.getDeviceClass())
    #print("\n")

  except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))
    #print("Press Enter to Exit...\n")
    readin = sys.stdin.read(1)
    exit(1)   
  
def DCMotorDetached(e):
  detached = e
  try:
    detached_port = detached.getHubPort()
    detached_channel = detached.getChannel()
    #print("\nDetach event on Port %d Channel %d" % (detached_port, detached_channel))
  except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))
    print("Press Enter to Exit...\n")
    readin = sys.stdin.read(1)
    exit(1)   

def ErrorEvent(e, eCode, description):
  print("Error %i : %s" % (eCode, description))

def VelocityUpdateHandler(e, velocity):
  global motor_velocity
  motor_velocity = velocity
  #print("Velocity: %f" % velocity)

try:
  ch_motor.setOnAttachHandler(DCMotorAttached)
  ch_motor.setOnDetachHandler(DCMotorDetached)
  ch_motor.setOnErrorHandler(ErrorEvent)

  ch_motor.setOnVelocityUpdateHandler(VelocityUpdateHandler)

  #print("Waiting for the Phidget DCMotor Object to be attached...")
  ch_motor.openWaitForAttachment(5000)
except PhidgetException as e:
  print("Phidget Exception %i: %s" % (e.code, e.details))
  print("Press Enter to Exit...\n")
  readin = sys.stdin.read(1)
  exit(1)


# Set up encoder

try:
  ch_encoder = Encoder()
except RuntimeError as e:
  print("Runtime Exception %s" % e.details)
  print("Press Enter to Exit...\n")
  readin = sys.stdin.read(1)
  exit(1)

def EncoderAttached(self):
  try:
    attached = self
    #print("\nAttach Event Detected (Information Below)")
    #print("===========================================")
    #print("Library Version: %s" % attached.getLibraryVersion())
    #print("Serial Number: %d" % attached.getDeviceSerialNumber())
    #print("Channel: %d" % attached.getChannel())
    #print("Channel Class: %s" % attached.getChannelClass())
    #print("Channel Name: %s" % attached.getChannelName())
    #print("Device ID: %d" % attached.getDeviceID())
    #print("Device Version: %d" % attached.getDeviceVersion())
    #print("Device Name: %s" % attached.getDeviceName())
    #print("Device Class: %d" % attached.getDeviceClass())
    #print("\n")

  except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))
    print("Press Enter to Exit...\n")
    readin = sys.stdin.read(1)
    exit(1)   
  
def EncoderDetached(self):
  detached = self
  try:
    detached_port = detached.getHubPort()
    detached_channel = detached.getChannel()
    #print("\nDetach event on Port %d Channel %d" % (detached_port, detached_channel))
  except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))
    print("Press Enter to Exit...\n")
    readin = sys.stdin.read(1)
    exit(1)   

def ErrorEvent(self, eCode, description):
  print("Error %i : %s" % (eCode, description))

def PositionChangeHandler(self, positionChange, timeChange, indexTriggered):
  global motor_count
  global focus_position
  global focus_pix
  global focus_arcsec
  
  # Update the encoder count and focus values
  
  motor_count += positionChange
  focus_position = motor_count/counts_per_micron
  focus_pix = abs(motor_count)/counts_per_pixel
  focus_arcsec = abs(motor_count)/counts_per_arcsec

try:
  ch_encoder.setOnAttachHandler(EncoderAttached)
  ch_encoder.setOnDetachHandler(EncoderDetached)
  ch_encoder.setOnErrorHandler(ErrorEvent)
  ch_encoder.setOnPositionChangeHandler(PositionChangeHandler)
  ch_encoder.openWaitForAttachment(5000)
except PhidgetException as e:
  print("Phidget Exception %i: %s" % (e.code, e.details))
  print("Press Enter to Exit...\n")
  readin = sys.stdin.read(1)
  exit(1)

if (not ch_encoder.getEnabled()):
  ch_encoder.setEnabled(1)

#
# Done with set up of parameters, file, motor and encoder
#


# Request a change in the focus setting 
# Return the count change achieved

def change_focus(delta_count):

  # Test for acceptable request and return with no action if out of bounds

  if (abs(delta_count)  > delta_count_max):
    #Request exceeds 1000 microns (1 mm) limit.  Use a smaller increment.
    return(0)
 
  start_count = motor_count
  
  # Turn on the motor with no motion
  # Acceleration is duty cycles per second per second from 0.5 to 50.
  # Velocity is fraction of duty cycle from -1. to + 1.

  try:
    ch_motor.setAcceleration(2.)
    ch_motor.setTargetVelocity(0.)
    sleep(1) 
  except PhidgetException as e:
    print("Focuser motor Phidget exception %i: %s" % (e.code, e.details))
    print("Exiting ...")
    exit(1)

  delay = abs(delta_count)/counts_per_second

  if (delta_count > 0.):
    try:
      ch_motor.setTargetVelocity(0.9)
      sleep(delay) 
      ch_motor.setTargetVelocity(0.0)
      sleep(1) 
    except PhidgetException as e:
      print("Focuser motor Phidget exception %i: %s" % (e.code, e.details))

  if (delta_count < 0.):
    try:
      ch_motor.setTargetVelocity(-0.9)
      sleep(delay) 
      ch_motor.setTargetVelocity(0.0)
      sleep(1) 
    except PhidgetException as e:
      print("Focuser motor Phidget exception %i: %s" % (e.code, e.details))

  # Write the status file with the encoder count

  focus_status_fp = open(focus_status_file,'w')
  focus_status_fp.write("%7.0f \n" % motor_count)
  focus_status_fp.close()
  
  stop_count = motor_count
  actual_delta_count = stop_count - start_count
  
  return(actual_delta_count)

# Process command line request

if diagnostics:
  print ("Initial encoder count: ", format(motor_count,'7.0f'))
  print ("Initial focuser position (microns): ", format(focus_position, '7.0f'))


focus_change_pix = input_change_pix
dc_request = int(focus_change_pix*counts_per_pixel)
dc_result = change_focus(dc_request)

if diagnostics:
  print ("Requested change (counts): ", format(dc_request, '7.0f'))
  print ("Actual change (counts): ", format(dc_result, '7.0f'))
  print ("Final encoder count: ", format(motor_count,'7.0f'))

print ("Position (microns): ", format(focus_position,'7.0f') )
print ("Image (pix): ", format(focus_pix,'7.0f') )
print ("Sky (arcsec): ", format(focus_arcsec,'7.1f') )


# Clean up and exit

try:
    ch_motor.close()
except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))
    print("Press Enter to Exit...\n")
    readin = sys.stdin.read(1)
    exit(1) 

try:
    ch_encoder.close()
except PhidgetException as e:
    print("Phidget Exception %i: %s" % (e.code, e.details))
    print("Press Enter to Exit...\n")
    readin = sys.stdin.read(1)
    exit(1) 

exit(0)
